@@ -35,7 +35,7 @@
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/Boolean.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 #include <Lum/Model/String.h>
 #include <Lum/Model/Table.h>
 
@@ -63,57 +63,22 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdRefTable<Hiscore::Entry,std::vector<Hiscore::Entry> > HiscoreEntriesModel;
 typedef Lum::Base::Reference<HiscoreEntriesModel>                            HiscoreEntriesModelRef;
 
-class HiscoreEntryObject : public Lum::Object
+class HiscoreEntriesModelPainter : public Lum::TableCellPainter
 {
-private:
-  Lum::OS::FontRef font;
-  Hiscore::Entry   entry;
-
 public:
-  HiscoreEntryObject()
-  : font(Lum::OS::display->GetFont())
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    // no code
-  }
-
-  void SetEntry(const Hiscore::Entry& entry)
-  {
-    this->entry=entry;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    const Hiscore::Entry entry=dynamic_cast<const HiscoreEntriesModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of User
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     entry.name);
+    DrawStringLeftAligned(draw,
+                          x,y,width,height,
+                          entry.name);
+
 
     //
     //
@@ -128,45 +93,9 @@ public:
       time+=L" seconds";
     }
 
-    draw->DrawString(this->x+width-font->StringWidth(time),
-                     this->y+font->ascent,
-                     time);
-
-    draw->PopFont();
-    draw->PopForeground();
-  }
-
-  static HiscoreEntryObject* Create(bool horizFlex, bool vertFlex)
-  {
-    HiscoreEntryObject *object=new HiscoreEntryObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class HiscoreEntryDataProvider : public HiscoreEntriesModel::DataProvider
-{
-private:
-  HiscoreEntryObject *object;
-
-public:
-  HiscoreEntryDataProvider()
-   : object(HiscoreEntryObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~HiscoreEntryDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const HiscoreEntriesModel::Iterator& iter, size_t column) const
-  {
-    object->SetEntry(*iter);
-    return object;
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           time);
   }
 };
 
@@ -184,7 +113,7 @@ public:
                     sweeper->GetAreaHeight(),
                     sweeper->GetMines());
 
-    list=new HiscoreEntriesModel(score->list,new HiscoreEntryDataProvider());
+    list=new HiscoreEntriesModel(score->list);
     list->SetEmptyText(L"(No hiscores)");
   }
 
@@ -201,13 +130,13 @@ public:
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetMinWidth(Lum::Base::Size::stdCharWidth,30);
-    table->SetMinHeight(Lum::Base::Size::stdCharHeight,10);
     table->SetShowHeader(false);
     table->GetTableView()->SetAutoFitColumns(true);
+    table->GetTableView()->SetAutoHSize(true);
     table->GetTableView()->SetAutoVSize(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
+    table->SetPainter(new HiscoreEntriesModelPainter());
     panel->Add(table);
 
     return panel;
@@ -317,57 +246,21 @@ public:
 typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
 typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
 
-class ThemeObject : public Lum::Object
+class ThemesModelPainter : public Lum::TableCellPainter
 {
-private:
-  Lum::OS::FontRef font;
-  Theme            theme;
-
 public:
-  ThemeObject()
-  : font(Lum::OS::display->GetFont())
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    // no code
-  }
-
-  void SetTheme(const Theme& theme)
-  {
-    this->theme=theme;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    const Theme theme=dynamic_cast<const ThemesModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of theme
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     theme.name);
+    DrawStringLeftAligned(draw,
+                          x,y,width,height,
+                          theme.name);
 
     //
     //
@@ -375,45 +268,9 @@ public:
 
     std::wstring size=Lum::Base::NumberToWString(theme.boxSize)+L" pixel";
 
-    draw->DrawString(this->x+width-font->StringWidth(size),
-                     this->y+font->ascent,
-                     size);
-
-    draw->PopFont();
-    draw->PopForeground();
-  }
-
-  static ThemeObject* Create(bool horizFlex, bool vertFlex)
-  {
-    ThemeObject *object=new ThemeObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class ThemeDataProvider : public ThemesModel::DataProvider
-{
-private:
-  ThemeObject *object;
-
-public:
-  ThemeDataProvider()
-  : object(ThemeObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~ThemeDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const ThemesModel::Iterator& iter, size_t column) const
-  {
-    object->SetTheme(*iter);
-    return object;
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           size);
   }
 };
 
@@ -428,7 +285,7 @@ private:
 public:
   SelectTheme()
   : okAction(new Lum::Model::Action),
-    list(new ThemesModel(themes,new ThemeDataProvider())),
+    list(new ThemesModel(themes,NULL)),
     selection(new Lum::Model::SingleLineSelection),
     result(L"")
   {
@@ -447,8 +304,6 @@ public:
     Lum::Model::HeaderRef headerModel;
 
     panel=Lum::VPanel::Create(true,true);
-    panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
-    panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
     headerModel->AddColumn(L"Theme",Lum::Base::Size::stdCharWidth,20,true);
@@ -460,7 +315,11 @@ public:
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
-    table->SetDoubleClickAction(okAction);
+    table->SetDefaultAction(okAction);
+    table->SetPainter(new ThemesModelPainter());
+    table->GetTableView()->SetAutoFitColumns(true);
+    table->GetTableView()->SetAutoHSize(true);
+    table->GetTableView()->SetAutoVSize(true);
     panel->Add(table);
 
     return panel;
@@ -625,21 +484,19 @@ public:
       mines->Set(Lum::Base::RoundDiv(99*100,30*16));
     }
     else if (model==themeAction && themeAction->IsFinished()) {
-      SelectTheme *dialog=new SelectTheme();
+      SelectTheme dialog;
 
-      dialog->SetParent(this);
-      if (dialog->Open()) {
-        dialog->SetExitAction(dialog->GetClosedAction());
-        dialog->EventLoop();
-        dialog->Close();
+      dialog.SetParent(this);
+      if (dialog.Open()) {
+        dialog.SetExitAction(dialog.GetClosedAction());
+        dialog.EventLoop();
+        dialog.Close();
       }
 
-      if (!dialog->GetResult().empty()) {
-        ::themeName=dialog->GetResult();
-        themeName->Set(dialog->GetResult());
+      if (!dialog.GetResult().empty()) {
+        ::themeName=dialog.GetResult();
+        themeName->Set(dialog.GetResult());
       }
-
-      delete dialog;
     }
     else {
       Dialog::Resync(model,msg);
@@ -804,47 +661,41 @@ public:
 
   void ShowSettings()
   {
-    Settings* settings;
+    Settings settings(sweeper);
 
-    settings=new Settings(sweeper);
-    settings->SetParent(this);
-    if (settings->Open()) {
-      settings->SetExitAction(settings->GetClosedAction());
-      settings->EventLoop();
-      settings->Close();
+    settings.SetParent(this);
+    if (settings.Open()) {
+      settings.SetExitAction(settings.GetClosedAction());
+      settings.EventLoop();
+      settings.Close();
 
-      if (sweeper->SetSize(settings->horiz->Get(),
-                           settings->vert->Get(),
-                           Lum::Base::RoundDiv(settings->horiz->Get()*settings->vert->Get()*settings->mines->Get(),100))) {
+      if (sweeper->SetSize(settings.horiz->Get(),
+                           settings.vert->Get(),
+                           Lum::Base::RoundDiv(settings.horiz->Get()*settings.vert->Get()*settings.mines->Get(),100))) {
         SetCurrentGame(sweeper->GetAreaWidth(),
                        sweeper->GetAreaHeight(),
                        sweeper->GetMines());
-        SetOneButtonMode(settings->oneButtonMode->Get());
+        SetOneButtonMode(settings.oneButtonMode->Get());
 
-        sweeper->SetOneButtonMode(settings->oneButtonMode->Get());
+        sweeper->SetOneButtonMode(settings.oneButtonMode->Get());
         sweeper->SetThemeName(themeName);
         sweeper->Run();
       }
 
     }
-
-    delete settings;
   }
 
   void ShowHiscore()
   {
-    ScoreView* score;
+    ScoreView score(sweeper);
 
-    score=new ScoreView(sweeper);
-    score->SetTitle(L"Hiscore");
-    score->SetParent(this);
-    if (score->Open()) {
-      score->SetExitAction(score->GetClosedAction());
-      score->EventLoop();
-      score->Close();
+    score.SetTitle(L"Hiscore");
+    score.SetParent(this);
+    if (score.Open()) {
+      score.SetExitAction(score.GetClosedAction());
+      score.EventLoop();
+      score.Close();
     }
-
-    delete score;
   }
 
   void ShowHiscoreEdit(size_t pos)
